/* ////////////////////////////////////////////////////////////

File Name: yswizard.h
Copyright (c) 2015 Soji Yamakawa.  All rights reserved.
http://www.ysflight.com

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//////////////////////////////////////////////////////////// */

#ifndef YSWIZARD_IS_INCLUDED
#define YSWIZARD_IS_INCLUDED
/* { */

#include "ysshell.h"

// Declaration /////////////////////////////////////////////
class YsWizard
{
public:
	static const char *ClassName;
	virtual const char *WhatItIs(void){return YsWizard::ClassName;}
};

// Declaration /////////////////////////////////////////////
class YsCollisionOfPolygon : public YsWizard
{
public:
	static const char *ClassName;
	virtual const char *WhatItIs(void){return YsCollisionOfPolygon::ClassName;}

	YsCollisionOfPolygon();

	int GetNumVertexOfPolygon1(void) const;
	const YsVec3 *GetVertexListOfPolygon1(void) const;
	int GetNumVertexOfPolygon2(void) const;
	const YsVec3 *GetVertexListOfPolygon2(void) const;

	void SetPolygon1(YSSIZE_T nv1,const YsVec3 v1[]);
	void SetPolygon2(YSSIZE_T nv2,const YsVec3 v2[]);
	YSRESULT PrecomputeProjectionOfPolygon1(void);
	YSRESULT PrecomputeProjectionOfPolygon2(void);
	void SetPolygon1(YSSIZE_T nv1,const YsVec3 v1[],const YsVec3 &nom1);
	void SetPolygon2(YSSIZE_T nv2,const YsVec3 v2[],const YsVec3 &nom2);
	YSBOOL CheckOnTheSamePlane(void) const;
	YSBOOL CheckCollision(void) const;  // Just check true/false
	YSBOOL CheckCollision(YsVec3 &firstFound) const;  // Just check true/false
	YSINTERSECTION CheckPenetration(void) const;  // Check aprt/touch/penetrate

	YSBOOL OneEdgeLiesOnTheOtherPlane(void) const;

	YSBOOL CheckIntersectionBetweenPolygon1AndBlock(const YsVec3 &min,const YsVec3 &max) const;
	YSBOOL CheckIntersectionBetweenPolygon1AndLineSegment(const YsVec3 &p1,const YsVec3 &p2) const;
	YSBOOL CheckIntersectionBetweenPolygon1AndLineSegment(YsVec3 &crs,const YsVec3 &p1,const YsVec3 &p2) const;  // 2008/11/06

protected:
	YSSIZE_T np1;
	const YsVec3 *p1;
	YsVec3 p1min,p1max;
	YsPlane pln1;

	YsArray <YsVec2,16> p1Prj;
	YsVec2 p1PrjMin,p1PrjMax;
	YsMatrix4x4 p1PrjMat;

	YSSIZE_T np2;
	const YsVec3 *p2;
	YsVec3 p2min,p2max;
	YsPlane pln2;

	YsArray <YsVec2,16> p2Prj;
	YsVec2 p2PrjMin,p2PrjMax;
	YsMatrix4x4 p2PrjMat;

	YSSIDE CheckInsideOfPolygon1(const YsVec3 &tst) const;
	YSSIDE CheckInsideOfPolygon2(const YsVec3 &tst) const;
};

// Declaration /////////////////////////////////////////////
class YsSwordPolygonAttr
{
public:
	YsSwordPolygonAttr();
	YSBOOL isOverlappingPolygon;  // For finding overlapping polygon
	int noteOverlapping;          // for polygon ID or any purpose
};

class YsSwordPolygon
{
public:
	YsSwordPolygonAttr attrSet;

	YsArray <YsVec3> plg;  // Original Polygon
	YsArray <YsVec2> prj;  // Projected Polygon
	YsArray <int> vtId;    // Corresponding Vertex Id (optional)
	YsArray <YsShellVertexHandle> vtHd;  // Corresponding Vertex Handle (optional)
};

// Declaration /////////////////////////////////////////////
class YsSwordNewVertexLog
{
public:
	YsSwordNewVertexLog();

	int freeAttribute;
	int betweenVt1,betweenVt2;
	int idCreated;
	YsVec3 pos;
};

// Declaration /////////////////////////////////////////////

class YsSword : public YsWizard
{
private:
	const YsSword &operator=(const YsSword &from);  // Don't use copy operator
	YsSword(const YsSword &from); // Don't use copy constructor
public:
	static const char *ClassName;
	virtual const char *WhatItIs(void){return YsSword::ClassName;}

	YsSword();
	~YsSword();

	void Encache(void) const;

	int GetNumPolygon(void) const;
	int GetNumVertexOfPolygon(YSSIZE_T id) const;
	const YsArray <YsVec3> *GetPolygon(YSSIZE_T id) const;
	const YsArray <int> *GetVertexIdList(YSSIZE_T id) const;
	const YsArray <YsShell::VertexHandle> *GetVtHdList(YSSIZE_T id) const;

	int GetNumNewVertex(void) const;
	const YsSwordNewVertexLog *GetNewVertexLog(int i) const;

	// For 3D Operation (Slash by plane)
	YSRESULT SetInitialPolygon(YSSIZE_T np,const YsVec3 p[]);
	YSRESULT SetInitialPolygon(YSSIZE_T np,const YsVec3 p[],const int id[]);
	YSRESULT SetInitialPolygon(YSSIZE_T np,const YsVec3 p[],const int id[],const YsShell::VertexHandle vtHd[]);

public:
	YSRESULT Slash(const YsPlane &pln);
	YSRESULT Slash(YSSIZE_T nv,const YsVec3 v[]);
	YSRESULT SlashByOverlappingPolygon3(YSSIZE_T nv,const YsVec3 v[],int note=0);
	YSBOOL IsFromOverlappingPolygon3(YSSIZE_T i) const;
	YSBOOL IsFromOverlappingPolygon3(YSSIZE_T i,int &note) const;
	YSRESULT GetVertexListOfPolygon(YsVec3 vec[],YSSIZE_T maxcount,int id) const;
	YSRESULT GetVertexListOfPolygon(const YsVec3 *vec[],YSSIZE_T maxcount,int id) const;

	// For 2D Operation (Slash by line)
	YSRESULT SetInitialPolygon(YSSIZE_T np,const YsVec2 p[],const int id[]=NULL);
	YSRESULT Slash(const YsVec2 &org,const YsVec2 &vec);
	YSRESULT GetVertexListOfPolygon(YsVec2 vec[],YSSIZE_T maxcount,int id) const;

	// For both
	YSRESULT Triangulate(YSCONVEXNIZESTRATEGY strategy=YSCONVEXNIZEDEFAULT);
	YSRESULT Convexnize(YSCONVEXNIZESTRATEGY strategy=YSCONVEXNIZEDEFAULT);

	// Update Ids
	YSRESULT UpdateVertexId(int oldId,int newId);

	YSBOOL debugMode;

protected:
	YsList <YsSwordPolygon> *SeparateByTwo(YsList <YsSwordPolygon> *org,int id1,int id2);

	YsList <YsSwordPolygon> *SeparateByMulti(YsList <YsSwordPolygon> *org,YsArray <int> &cutPoint);


	YSRESULT SlashOnePolygon(YsList <YsSwordPolygon> *target,const YsPlane &pln,YSSIZE_T nv=0,const YsVec3 v[]=NULL);
	YSRESULT FindAddSlashPoint
	    (YsArray <int> &idx,YsList <YsSwordPolygon> *taget,const YsPlane &pln,
	     int nCuttingPolygonVtx=0,const YsVec3 *cuttingPolygon=NULL);
	YSRESULT FindAddSlashPointToNeighborPolygons
		(const YsVec3 &v1,const YsVec3 &v2,
		 const YsVec3 &newVtx3,const YsVec2 &newVtx2,int newCreatedId,YsList <YsSwordPolygon> *exclude);

	YSRESULT ConvexnizeGo(YSBOOL tri,YsList <YsSwordPolygon> *target,YSCONVEXNIZESTRATEGY strategy);
	YsList <YsSwordPolygon> *ConvexnizeOnePolygon
	    (YSBOOL tri,YsList <YsSwordPolygon> *target,YSCONVEXNIZESTRATEGY strategy);

	YSRESULT FindConvexnizeVertexPair(int &id1,int &id2,YsList <YsSwordPolygon> *target,YSCONVEXNIZESTRATEGY strategy);

	int newCreateId;
	YsMatrix4x4 plgPrjMatrix;
	YsList <YsSwordPolygon> *plg;
	YsList <YsSwordNewVertexLog> *cre;
};

// Declaration /////////////////////////////////////////////
class YsBoundingBoxMaker2 : public YsWizard
{
public:
	static const char *ClassName;
	virtual const char *WhatItIs(void){return YsBoundingBoxMaker2::ClassName;}

	YsBoundingBoxMaker2();

	void Begin(void);
	void Begin(const YsVec2 &vec);
	void Add(const YsVec2 &vec);
	void Make(YSSIZE_T n,const YsVec2 v[]);
	void Make(YsList <YsVec2> *lst);
	void Make(const YsArray <YsVec2> &ary);
	void Get(YsVec2 &min,YsVec2 &max) const;
	void Get(YsVec2 minmax[]) const;
	const YsVec2 &GetCenter(YsVec2 &cen) const;

	YSBOOL IsInside(const YsVec2 &pos) const;  // 2008/09/16
	YSBOOL IsInside(const YsVec3 &pos) const;  // 2008/09/16
	YsVec2 Center(void) const;                 // 2008/09/16

	YsVec2 GetDimension(void) const;

protected:
	YSBOOL first;
	YsVec2 min,max;
};

// Declaration /////////////////////////////////////////////
class YsBoundingBoxMaker3 : public YsWizard
{
public:
	static const char *ClassName;
	virtual const char *WhatItIs(void){return YsBoundingBoxMaker3::ClassName;}

	YsBoundingBoxMaker3();
	template <const int N>
	inline YsBoundingBoxMaker3(const YsArray <YsVec3,N> &ary);

	void Begin(void);
	void Begin(const YsVec3 &vec);

	void Add(const YsVec3 &vec);
	void Add(YSSIZE_T n,const YsVec3 v[]);
	template <const int N>
	void Add(const YsArray <YsVec3,N> &v);

	void Make(YSSIZE_T n,const YsVec3 v[]);
	void Make(const YsList <YsVec3> *lst);
	template <const int N>
	void Make(const YsArray <YsVec3,N> &ary);

	void Make(const YsShell &shl,YSSIZE_T nVt,const YsShellVertexHandle vtHd[]);
	void Get(YsVec3 &min,YsVec3 &max) const;
	void Get(YsVec3 minMax[2]) const;
	const YsVec3 &GetCenter(YsVec3 &cen) const;

	YSBOOL IsInside(const YsVec3 &pos) const;    // 2008/09/16
	YSBOOL IsInside(const YsVec2 &pos) const;    // 2008/09/16
	YSBOOL IsInsideXY(const YsVec3 &pos) const;  // 2008/09/16
	YSBOOL IsInsideXY(const YsVec2 &pos) const;  // 2008/09/16
	YsVec3 Center(void) const;                   // 2008/09/16

	YsVec3 GetDimension(void) const;

	/*! Returns the length of minimum to maximum. */
	const double GetDiagonalLength(void) const;

protected:
	YSBOOL first;
	YsVec3 min,max;
};

template <const int N>
inline YsBoundingBoxMaker3::YsBoundingBoxMaker3(const YsArray <YsVec3,N> &ary)
{
	first=YSTRUE;
	Make(ary);
}

template <const int N>
void YsBoundingBoxMaker3::Add(const YsArray <YsVec3,N> &v)
{
	return Add(v.GetN(),v);
}

template <const int N>
void YsBoundingBoxMaker3::Make(const YsArray <YsVec3,N> &ary)
{
	if(ary.GetNumItem()>0)
	{
		Begin(ary.GetItem(0));
	}
	int i;
	for(i=0; i<ary.GetNumItem(); i++)
	{
		Add(ary.GetItem(i));
	}
}


////////////////////////////////////////////////////////////

class YsLineSegmentTracer
{
public:
	YsVec3 pos;
	int seg;
	double segParam;

	const YsVec3 &GetPosition(void) const;
	int GetSegment(void) const;
};

class YsTraceLineSegment
{
protected:
	YSBOOL isLoop;
	YsArray <YsVec3> lSeg;

	YsLineSegmentTracer curPos;

	double totalLength;
	YsArray <double> segLength;
	YsArray <double> segLengthAddUp;

	YSRESULT ResetLineSegment(void);

public:
	YSRESULT SetLineSegment(YSSIZE_T np,const YsVec3 p[],YSBOOL isLoop);
	template <const int N>
	YSRESULT SetLineSegment(YsArray <YsVec3,N> &p,YSBOOL isLoop);

	int GetCurrentSegment(void) const;
	const YsVec3 &GetCurrentPosition(void) const;
	const YsVec3 GetCurrentTangent(void) const;
	const double &GetTotalLength(void) const;

	YSRESULT MoveByDistance(const double &dist);
	YSRESULT SetPositionByParameter(const double &t);

	YSRESULT MoveByDistance(YsLineSegmentTracer &tracer,const double &dist);
	YSRESULT SetPositionByParameter(YsLineSegmentTracer &tracer,const double &t);
	YSRESULT SetPositionByVertex(YsLineSegmentTracer &tracer,int idx);
	double GetParameter(const YsLineSegmentTracer &tracer) const;
};

template <const int N>
YSRESULT YsTraceLineSegment::SetLineSegment(YsArray <YsVec3,N> &p,YSBOOL islp)
{
	lSeg=p;
	isLoop=islp;
	if(islp==YSTRUE)
	{
		lSeg.Append(lSeg[0]);
	}
	return ResetLineSegment();
}


/* } */
#endif
